from typing import List, Any
from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck

class MDBPublicIP(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        name = "Ensure public IP is not assigned to database cluster."
        id = "CKV_YC_12"
        categories = [CheckCategories.NETWORKING]
        supported_resources = [
            "yandex_mdb_postgresql_cluster",
            "yandex_mdb_sqlserver_cluster",
            "yandex_mdb_mysql_cluster",
            "yandex_mdb_mongodb_cluster",
            "yandex_mdb_kafka_cluster",
            "yandex_mdb_greenplum_cluster",
            "yandex_mdb_elasticsearch_cluster",
            "yandex_mdb_clickhouse_cluster",
        ]
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
        )

    def get_inspected_key(self) -> str:
        if self.entity_type == "yandex_mdb_kafka_cluster":
            return "config/[0]/assign_public_ip"
        if self.entity_type == "yandex_mdb_greenplum_cluster":
            return "assign_public_ip"
        return "host/[0]/assign_public_ip"

    def get_forbidden_values(self) -> List[Any]:
        return [True]

check = MDBPublicIP()


from checkov.common.models.enums import CheckCategories
from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck


class EBSEncryption(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure all data stored in the EBS is securely encrypted"
        id = "CKV_AWS_3"
        supported_resources = ['AWS::EC2::Volume']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'Properties/Encrypted'

check = EBSEncryption()



import ast
from typing import Tuple, List, Union


def get_class_members_errors(model_parts_info, class_def: ast.ClassDef) -> List[Tuple[int, int, str]]:
    errors = []
    forbidden_types = ['field']
    if skip_dataclasses(class_def):
        return errors
    for model_part in model_parts_info:
        if model_part['type'] in forbidden_types:
            node_name = get_node_name(model_part['node'], model_part['type'])
            errors.append((model_part['node'].lineno, model_part['node'].col_offset, f"CCE003 Class level {model_part['type']} '{node_name}' detected in class {model_part['model_name']}",))
    return errors


def skip_dataclasses(class_def: ast.ClassDef) -> bool:
    if class_def.decorator_list is not None:
        for decorator in class_def.decorator_list:
            if not isinstance(decorator, ast.Load):
                return True
            decorator: ast.Load
            if decorator.id == 'dataclass':
                return True
    return False


def get_node_name(node, node_type: str):
    special_methods_names = (
        '__new__',
        '__init__',
        '__post_init__',
        '__str__',
        'save',
        'delete',
    )
    name_getters_by_type = [
        ('docstring', lambda n: 'docstring'),
        ('meta_class', lambda n: 'Meta'),
        ('constant', lambda n: n.target.id if isinstance(n, ast.AnnAssign) else n.targets[0].id),  # type: ignore
        ('field', get_name_for_field_node_type),
        (('method',) + special_methods_names, lambda n: n.name),
        ('nested_class', lambda n: n.name),
        ('expression', lambda n: '<class_level_expression>'),
        ('if', lambda n: 'if ...'),
    ]
    for type_postfix, name_getter in name_getters_by_type:
        if node_type.endswith(type_postfix):  # type: ignore
            return name_getter(node)


def get_name_for_field_node_type(node: Union[ast.Assign, ast.AnnAssign]) -> str:
    name = '<class_level_assignment>'
    if isinstance(node, ast.AnnAssign):
        name = node.target.id if isinstance(node.target, ast.Name) else name
    elif isinstance(node.targets[0], ast.Name):
        name = node.targets[0].id
    elif hasattr(node.targets[0], 'attr'):
        name = node.targets[0].attr  # type: ignore
    elif isinstance(node.targets[0], ast.Tuple):
        name = ', '.join([e.id for e in node.targets[0].elts if isinstance(e, ast.Name)])

    return name



from abc import abstractmethod
from checkov.common.bridgecrew.run_metadata.registry import registry
from checkov.common.bridgecrew.run_metadata.ci_variables import (
    BC_TO_BRANCH,
    BC_PR_ID,
    BC_PR_URL,
    BC_COMMIT_HASH,
    BC_COMMIT_URL,
    BC_AUTHOR_NAME,
    BC_AUTHOR_URL,
    BC_RUN_ID,
    BC_RUN_URL,
    BC_REPOSITORY_URL,
    BC_FROM_BRANCH,
)


class RunMetaDataExtractor:
    from_branch = ""
    to_branch = ""
    pr_id = ""
    pr_url = ""
    commit_hash = ""
    commit_url = ""
    author_name = ""
    author_url = ""
    run_id = ""
    run_url = ""
    repository_url = ""

    def __init__(self, from_branch,
                 to_branch,
                 pr_id,
                 pr_url,
                 commit_hash,
                 commit_url,
                 author_name,
                 author_url,
                 run_id,
                 run_url,
                 repository_url):
        self.from_branch = from_branch
        self.to_branch = to_branch
        self.pr_id = pr_id
        self.pr_url = pr_url
        self.commit_hash = commit_hash
        self.commit_url = commit_url
        self.author_name = author_name
        self.author_url = author_url
        self.run_id = run_id
        self.run_url = run_url
        self.repository_url = repository_url
        self.override_metadata_from_env_variables()
        registry.register(extractor=self)

    def override_metadata_from_env_variables(self):
        if BC_FROM_BRANCH:
            self.from_branch = BC_FROM_BRANCH
        if BC_TO_BRANCH:
            self.to_branch = BC_TO_BRANCH
        if BC_PR_ID:
            self.pr_id = BC_PR_ID
        if BC_PR_URL:
            self.pr_url = BC_PR_URL
        if BC_COMMIT_HASH:
            self.commit_hash = BC_COMMIT_HASH
        if BC_COMMIT_URL:
            self.commit_url = BC_COMMIT_URL
        if BC_AUTHOR_NAME:
            self.author_name = BC_AUTHOR_NAME
        if BC_AUTHOR_URL:
            self.author_url = BC_AUTHOR_URL
        if BC_RUN_ID:
            self.run_id = BC_RUN_ID
        if BC_RUN_URL:
            self.run_url = BC_RUN_URL
        if BC_REPOSITORY_URL:
            self.repository_url = BC_REPOSITORY_URL

    @abstractmethod
    def is_current_ci(self):
        pass



from typing import Dict, Any

from checkov.common.models.enums import CheckResult
from checkov.kubernetes.checks.resource.base_container_check import BaseK8sContainerCheck


class KubeletProtectKernelDefaults(BaseK8sContainerCheck):
    def __init__(self) -> None:
        # CIS-1.6 4.2.6
        id = "CKV_K8S_144"
        name = "Ensure that the --protect-kernel-defaults argument is set to true"
        super().__init__(name=name, id=id)

    def scan_container_conf(self, metadata: Dict[str, Any], conf: Dict[str, Any]) -> CheckResult:
        self.evaluated_container_keys = ["command"]
        if conf.get("command"):
            if "kubelet" in conf["command"]:
                if "--protect-kernel-defaults=true" not in conf["command"]:
                    return CheckResult.FAILED

        return CheckResult.PASSED


check = KubeletProtectKernelDefaults()



import logging
import platform
import stat
from abc import abstractmethod, ABC
from pathlib import Path
from typing import Dict, Any, List
from datetime import datetime, timedelta

import aiohttp
from checkov.common.bridgecrew.platform_integration import bc_integration, BcPlatformIntegration
from checkov.common.util.data_structures_utils import merge_dicts
from checkov.common.util.http_utils import get_default_post_headers, request_wrapper


class TwistcliIntegration(ABC):
    vulnerabilities_base_path = "/api/v1/vulnerabilities"

    def get_bc_api_key(self) -> str:
        return bc_integration.get_auth_token()

    def get_proxy_address(self) -> str:
        return f"{bc_integration.api_url}{self.vulnerabilities_base_path}/docker-images/twistcli/proxy"

    def download_twistcli(self, cli_file_name: Path) -> None:
        # backwards compatibility, should be removed in a later stage
        cli_file_name_path = cli_file_name if isinstance(cli_file_name, Path) else Path(cli_file_name)

        os_type = platform.system().lower()

        response = request_wrapper("GET",
                                   f"{bc_integration.api_url}{self.vulnerabilities_base_path}/twistcli?os={os_type}",
                                   headers=bc_integration.get_default_headers("GET"),
                                   should_call_raise_for_status=True)

        cli_file_name_path.write_bytes(response.content)
        cli_file_name_path.chmod(cli_file_name_path.stat().st_mode | stat.S_IEXEC)
        logging.debug("twistcli downloaded and has execute permission")

    # can be removed, if image scanning is also using report_results_asyncio
    # make sure to remove it only after validating it isn't used from the platform
    def report_results(self, twistcli_scan_result: Dict[str, Any], file_path: Path, **kwargs: Any) -> None:
        payload = self.create_report(
            twistcli_scan_result=twistcli_scan_result,
            bc_platform_integration=bc_integration,
            file_path=file_path,
            **kwargs,
        )

        request_wrapper("POST", f"{bc_integration.api_url}{self.vulnerabilities_base_path}/results",
                        headers=bc_integration.get_default_headers("POST"),
                        json=payload, should_call_raise_for_status=True)

    async def report_results_async(
        self,
        twistcli_scan_result: Dict[str, Any],
        bc_platform_integration: BcPlatformIntegration,
        bc_api_key: str,
        file_path: Path,
        **kwargs: Any,
    ) -> int:
        logging.info(f"Start to send report for package file {file_path}")

        payload = self.create_report(
            twistcli_scan_result=twistcli_scan_result,
            bc_platform_integration=bc_platform_integration,
            file_path=file_path,
            **kwargs,
        )
        headers = merge_dicts(
            get_default_post_headers(bc_platform_integration.bc_source, bc_platform_integration.bc_source_version),
            {"Authorization": bc_api_key},
        )

        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(resolver=aiohttp.AsyncResolver())) as session:
            async with session.post(
                url=f'{bc_integration.api_url}{self.vulnerabilities_base_path}/results', headers=headers, json=payload
            ) as response:
                content = await response.text()

            if response.ok:
                logging.info(f"Successfully send report for package file {file_path}")
                return 0
            else:
                logging.error(f"Failed to send report for package file {file_path}")
                logging.error(f"Status code: {response.status}, Reason: {response.reason}, Content: {content}")
                return 1

    @abstractmethod
    def create_report(self, **kwargs: Any) -> Dict[str, Any]:
        pass

    @staticmethod
    def get_vulnerabilities_for_report(scan_results: Dict[str, Any]) -> List[Dict[str, Any]]:
        return [
            {
                "cveId": vul.get("id"),
                "status": vul.get("status", "open"),
                "severity": vul.get("severity"),
                "packageName": vul.get("packageName"),
                "packageVersion": vul.get("packageVersion"),
                "link": vul.get("link"),
                "cvss": vul.get("cvss"),
                "vector": vul.get("vector"),
                "description": vul.get("description"),
                "riskFactors": vul.get("riskFactors"),
                "publishedDate": vul.get("publishedDate") or
                                 (datetime.now() - timedelta(days=vul.get("publishedDays", 0))).isoformat(),
            }
            for vul in scan_results.get("vulnerabilities") or []
        ]

    @staticmethod
    def get_packages_for_report(scan_results: Dict[str, Any]) -> List[Dict[str, Any]]:
        return [
            {
                "type": package.get("type"),
                "name": package.get("name"),
                "version": package.get("version"),
                "licenses": package.get("licenses") or [],
            }
            for package in scan_results.get("packages") or []
        ]


from __future__ import annotations

from abc import abstractmethod
from pathlib import Path
from typing import Type, Any, TYPE_CHECKING

from checkov.common.graph.db_connectors.db_connector import DBConnector
from checkov.common.graph.graph_builder.local_graph import LocalGraph

if TYPE_CHECKING:
    import networkx as nx
    from checkov.terraform.parser import Parser


class GraphManager:
    def __init__(self, db_connector: DBConnector[nx.DiGraph], parser: Parser | None, source: str = "") -> None:
        self.db_connector = db_connector
        self.source = source
        self.parser = parser

    @abstractmethod
    def build_graph_from_source_directory(
        self,
        source_dir: str,
        render_variables: bool = True,
        local_graph_class: Type[LocalGraph] = LocalGraph,
        parsing_errors: dict[str, Exception] | None = None,
        download_external_modules: bool = False,
        excluded_paths: list[str] | None = None,
    ) -> tuple[LocalGraph, dict[str, dict[str, Any]]]:
        pass

    @abstractmethod
    def build_graph_from_definitions(
        self, definitions: dict[str | Path, dict[str, Any]], render_variables: bool = True
    ) -> LocalGraph:
        pass

    def save_graph(self, graph: LocalGraph) -> nx.DiGraph:
        return self.db_connector.save_graph(graph)

    def get_reader_endpoint(self) -> nx.DiGraph:
        return self.db_connector.get_reader_endpoint()

    def get_writer_endpoint(self) -> nx.DiGraph:
        return self.db_connector.get_writer_endpoint()

    def disconnect_from_db(self) -> None:
        self.db_connector.disconnect()



from typing import Dict, List, Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class AKSUsesAzurePoliciesAddon(BaseResourceCheck):
    def __init__(self):
        name = "Ensure that AKS uses Azure Policies Add-on"
        id = "CKV_AZURE_116"
        supported_resources = ['azurerm_kubernetes_cluster']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        # since Azure provider v2.97.0
        azure_policy_enabled = conf.get("azure_policy_enabled", [None])[0]
        if azure_policy_enabled:
            self.evaluated_keys = ["azure_policy_enabled"]
            return CheckResult.PASSED
        # up to and including Azure provider v2.96.0
        self.evaluated_keys = ["addon_profile/[0]/azure_policy/[0]/enabled"]
        addon_profile = conf.get("addon_profile", [None])[0]
        if addon_profile and isinstance(addon_profile, dict):
            azure_policy = addon_profile.get("azure_policy", [None])[0]
            if azure_policy and isinstance(azure_policy, dict) and azure_policy.get("enabled", [None])[0]:
                return CheckResult.PASSED
        return CheckResult.FAILED


check = AKSUsesAzurePoliciesAddon()



from __future__ import annotations

import itertools
import logging
import os
import re
from abc import ABC, abstractmethod
from collections.abc import Iterable
from typing import List, Dict, Any, TYPE_CHECKING

from checkov.common.graph.checks_infra.base_check import BaseGraphCheck
from checkov.common.output.report import Report
from checkov.runner_filter import RunnerFilter

if TYPE_CHECKING:
    from checkov.common.checks_infra.registry import Registry
    from checkov.common.graph.checks_infra.registry import BaseRegistry
    from checkov.common.graph.graph_manager import GraphManager


def strtobool(val: str) -> int:
    """Convert a string representation of truth to true (1) or false (0).

    True values are 'y', 'yes', 't', 'true', 'on', and '1'; false values
    are 'n', 'no', 'f', 'false', 'off', and '0'.  Raises ValueError if
    'val' is anything else.
    """
    val = val.lower()
    if val in ('y', 'yes', 't', 'true', 'on', '1'):
        return 1
    elif val in ('n', 'no', 'f', 'false', 'off', '0'):
        return 0
    else:
        raise ValueError("invalid boolean value %r for environment variable CKV_IGNORE_HIDDEN_DIRECTORIES" % (val,))


CHECKOV_CREATE_GRAPH = strtobool(os.getenv("CHECKOV_CREATE_GRAPH", "True"))
IGNORED_DIRECTORIES_ENV = os.getenv("CKV_IGNORED_DIRECTORIES", "node_modules,.terraform,.serverless")
IGNORE_HIDDEN_DIRECTORY_ENV = strtobool(os.getenv("CKV_IGNORE_HIDDEN_DIRECTORIES", "True"))

ignored_directories = IGNORED_DIRECTORIES_ENV.split(",")


class BaseRunner(ABC):
    check_type = ""
    definitions = None
    context: dict[str, dict[str, Any]] | None = None
    breadcrumbs = None
    external_registries: list[BaseRegistry] | None = None
    graph_manager: GraphManager | None = None
    graph_registry: Registry | None = None

    def __init__(self, file_extensions: Iterable[str] | None = None, file_names: Iterable[str] | None = None):
        self.file_extensions = file_extensions or []
        self.file_names = file_names or []

    @abstractmethod
    def run(
            self,
            root_folder: str | None,
            external_checks_dir: list[str] | None = None,
            files: list[str] | None = None,
            runner_filter: RunnerFilter = RunnerFilter(),
            collect_skip_comments: bool = True,
    ) -> Report:
        pass

    def should_scan_file(self, filename: str) -> bool:
        # runners that are always applicable can do nothing and be included
        if not self.file_extensions and not self.file_names:
            return True

        basename = os.path.basename(filename)
        if basename and self.file_names and basename in self.file_names:
            return True

        extension = os.path.splitext(filename)[1]
        if extension and self.file_extensions and extension in self.file_extensions:
            return True

        return False

    def set_external_data(
            self,
            definitions: dict[str, dict[str, Any]] | None,
            context: dict[str, dict[str, Any]] | None,
            breadcrumbs: dict[str, dict[str, Any]] | None,
            **kwargs: Any,
    ) -> None:
        self.definitions = definitions
        self.context = context
        self.breadcrumbs = breadcrumbs

    def load_external_checks(self, external_checks_dir: List[str]) -> None:
        pass

    def get_graph_checks_report(self, root_folder: str, runner_filter: RunnerFilter) -> Report:
        pass

    def run_graph_checks_results(self, runner_filter: RunnerFilter) -> Dict[BaseGraphCheck, List[Dict[str, Any]]]:
        checks_results: Dict[BaseGraphCheck, List[Dict[str, Any]]] = {}

        if not self.graph_manager or not self.graph_registry:
            # should not happen
            logging.warning("Graph components were not initialized")
            return checks_results

        for r in itertools.chain(self.external_registries or [], [self.graph_registry]):
            r.load_checks()
            registry_results = r.run_checks(self.graph_manager.get_reader_endpoint(), runner_filter)
            checks_results = {**checks_results, **registry_results}
        return checks_results


def filter_ignored_paths(
    root_dir: str,
    names: list[str] | list[os.DirEntry[str]],
    excluded_paths: list[str] | None,
    included_paths: Iterable[str] | None = None
) -> None:
    # we need to handle legacy logic, where directories to skip could be specified using the env var (default value above)
    # or a directory starting with '.'; these look only at directory basenames, not relative paths.
    #
    # But then any other excluded paths (specified via --skip-path or via the platform repo settings) should look at
    # the path name relative to the root folder. These can be files or directories.
    # Example: take the following dir tree:
    # .
    #   ./dir1
    #      ./dir1/dir33
    #      ./dir1/.terraform
    #   ./dir2
    #      ./dir2/dir33
    #      /.dir2/hello.yaml
    #
    # if excluded_paths = ['dir1/dir33', 'dir2/hello.yaml'], then we would scan dir1, but we would skip its subdirectories. We would scan
    # dir2 and its subdirectory, but we'd skip hello.yaml.

    # first handle the legacy logic - this will also remove files starting with '.' but that's probably fine
    # mostly this will just remove those problematic directories hardcoded above.
    included_paths = included_paths or []
    for entry in list(names):
        path = entry.name if isinstance(entry, os.DirEntry) else entry
        if path in ignored_directories:
            safe_remove(names, entry)
        if path.startswith(".") and IGNORE_HIDDEN_DIRECTORY_ENV and path not in included_paths:
            safe_remove(names, entry)

    # now apply the new logic
    # TODO this is not going to work well on Windows, because paths specified in the platform will use /, and
    #  paths specified via the CLI argument will presumably use \\
    if excluded_paths:
        compiled = [re.compile(p.replace(".terraform", r"\.terraform")) for p in excluded_paths]
        for entry in list(names):
            path = entry.name if isinstance(entry, os.DirEntry) else entry
            if any(pattern.search(os.path.join(root_dir, path)) for pattern in compiled):
                safe_remove(names, entry)


def safe_remove(names: list[Any], path: Any) -> None:
    if path in names:
        names.remove(path)



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class ImageTagFixed(BaseResourceCheck):

    def __init__(self):
        """
         You should avoid using the :latest tag when deploying containers in production
         as it is harder to track which version of the image is running
         and more difficult to roll back properly.
         """
        name = "Image Tag should be fixed - not latest or blank"
        id = "CKV_K8S_14"
        supported_resources = ["kubernetes_pod"]
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        spec = conf.get('spec', [None])[0]
        if isinstance(spec, dict) and spec.get("container"):
            containers = spec.get("container")
            for idx, container in enumerate(containers):
                if not isinstance(container, dict):
                    return CheckResult.UNKNOWN
                if container.get("image"):
                    name = container.get("image")[0]
                    if ":" in name:
                        if name.split(":")[1] in ("latest", ""):
                            self.evaluated_keys = [f'spec/[0]/container/[{idx}]/image']
                            return CheckResult.FAILED
                        continue
                    if "@" in name:
                        continue
                    self.evaluated_keys = [f'spec/[0]/container/[{idx}]/image']
                    return CheckResult.FAILED
                self.evaluated_keys = [f'spec/[0]/container/[{idx}]']
                return CheckResult.FAILED
            return CheckResult.PASSED
        return CheckResult.FAILED


check = ImageTagFixed()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class TillerService(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that the Tiller Service (Helm v2) is deleted"
        id = "CKV_K8S_44"
        supported_resources = ["kubernetes_service"]
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        if "metadata" in conf and isinstance(conf["metadata"], list):
            metadata = conf.get("metadata")[0]

            if metadata.get("labels") and isinstance(metadata.get("labels"), list) \
                    and isinstance(metadata.get("labels")[0], dict):
                labels = metadata.get("labels")[0]
                self.evaluated_keys = ["metadata/[0]/labels"]
                if labels.get("app") == "helm":
                    self.evaluated_keys = ["metadata/[0]/labels/[0]/app"]
                    return CheckResult.FAILED
                elif labels.get("name") == "tiller":
                    self.evaluated_keys = ["metadata/[0]/labels/[0]/name"]
                    return CheckResult.FAILED

        if "spec" not in conf:
            self.evaluated_keys = [""]
            return CheckResult.FAILED
        spec = conf['spec'][0]

        if spec.get('selector') and isinstance(spec.get('selector'), list):
            selector = spec.get('selector')[0]
            if selector and isinstance(selector, dict):
                for v in selector.values():
                    test = str(v).lower()
                    if 'tiller' in test:
                        self.evaluated_keys = ["spec/[0]/selector/app"]
                        return CheckResult.FAILED
            return CheckResult.PASSED
        return CheckResult.UNKNOWN


check = TillerService()



from pathlib import Path
from typing import cast, List, Tuple, Dict, Any

from checkov.common.util.suppression import collect_suppressions_for_report

from pycep.typing import BicepJson

BICEP_COMMENT = "//"
DEFINITIONS_KEYS_TO_PARSE = {"parameters": "parameters", "resources": "resources"}


def build_definitions_context(definitions: Dict[Path, BicepJson], definitions_raw: Dict[Path, List[Tuple[int, str]]]
                              ) -> Dict[str, Dict[str, Any]]:
    definitions_context: Dict[str, Dict[str, Any]] = {}
    for file_path_object, file_path_definitions in definitions.items():
        file_path = str(file_path_object)
        definitions_context[file_path] = {}
        for definition_attribute, resources in file_path_definitions.items():
            if definition_attribute not in DEFINITIONS_KEYS_TO_PARSE.values():
                continue
            definitions_context[file_path][definition_attribute] = {}
            # ignore mypy mismatched type warning since it can't resolve this type correctly
            for resource_key, resource_attributes in resources.items():  # type:ignore[attr-defined]
                definition_resource = {"start_line": resource_attributes["__start_line__"], "end_line": resource_attributes["__end_line__"]}

                if definition_attribute == DEFINITIONS_KEYS_TO_PARSE["resources"]:
                    definition_key = f"{resource_attributes['type']}.{resource_key}"
                    int_start_line = cast(int, definition_resource["start_line"])
                    int_end_line = cast(int, definition_resource["end_line"])
                    code_lines_for_suppressions_check = definitions_raw[file_path_object][int_start_line: int_end_line]
                    definition_resource['skipped_checks'] = collect_suppressions_for_report(code_lines=code_lines_for_suppressions_check)
                elif definition_attribute == DEFINITIONS_KEYS_TO_PARSE["parameters"]:
                    definition_key = resource_key
                    definition_resource["type"] = resource_attributes['type']

                start_line = resource_attributes["__start_line__"]
                end_line = resource_attributes["__end_line__"]

                # add resource comments to definition lines
                current_line = str.strip(definitions_raw[file_path_object][start_line - 1][1])
                while not current_line or current_line[0] == BICEP_COMMENT:
                    start_line -= 1
                    current_line = str.strip(definitions_raw[file_path_object][start_line - 1][1])

                # remove next resource comments from definition lines
                current_line = str.strip(definitions_raw[file_path_object][end_line - 1][1])
                while not current_line or current_line[0] == BICEP_COMMENT:
                    end_line -= 1
                    current_line = str.strip(definitions_raw[file_path_object][end_line - 1][1])

                definition_resource["code_lines"] = definitions_raw[file_path_object][start_line - 1: end_line]
                definitions_context[file_path][definition_attribute][definition_key] = definition_resource

    return definitions_context



import itertools
from typing import Optional, Tuple, Dict, List, Any

from checkov.common.parsers.node import DictNode, ListNode
from checkov.terraform.context_parsers.tf_plan import parse

simple_types = (str, int, float, bool)


def _is_simple_type(obj: Any) -> bool:
    if obj is None:
        return True
    if isinstance(obj, simple_types):
        return True
    return False


def _is_list_of_simple_types(obj: Any) -> bool:
    if not isinstance(obj, list):
        return False
    for i in obj:
        if not _is_simple_type(i):
            return False
    return True


def _is_list_of_dicts(obj: Any) -> bool:
    if not isinstance(obj, list):
        return False
    for i in obj:
        if isinstance(i, dict):
            return True
    return False


def _hclify(obj: DictNode, conf: Optional[DictNode] = None, parent_key: Optional[str] = None) -> Dict[str, List[Any]]:
    ret_dict = {}
    if not isinstance(obj, dict):
        raise Exception("this method receives only dicts")
    if hasattr(obj, "start_mark") and hasattr(obj, "end_mark"):
        obj["start_line"] = obj.start_mark.line
        obj["end_line"] = obj.end_mark.line
    for key, value in obj.items():
        if _is_simple_type(value) or _is_list_of_simple_types(value):
            if parent_key == "tags":
                ret_dict[key] = value
            else:
                ret_dict[key] = _clean_simple_type_list([value])

        if _is_list_of_dicts(value):
            child_list = []
            conf_val = conf.get(key, []) if conf else []
            for internal_val, internal_conf_val in itertools.zip_longest(value, conf_val):
                if isinstance(internal_val, dict):
                    child_list.append(_hclify(internal_val, internal_conf_val, parent_key=key))
            if key == "tags":
                ret_dict[key] = [child_list]
            else:
                ret_dict[key] = child_list
        if isinstance(value, dict):
            child_dict = _hclify(value, parent_key=key)
            if parent_key == "tags":
                ret_dict[key] = child_dict
            else:
                ret_dict[key] = [child_dict]
    if conf and isinstance(conf, dict):
        found_ref = False
        for conf_key in conf.keys() - obj.keys():
            ref = next((x for x in conf[conf_key].get("references", []) if not x.startswith(("var.", "local."))), None)
            if ref:
                ret_dict[conf_key] = [ref]
                found_ref = True
        if not found_ref:
            for value in conf.values():
                if isinstance(value, dict) and "references" in value.keys():
                    ret_dict["references_"] = value["references"]

    return ret_dict


def _prepare_resource_block(resource: DictNode, conf: Optional[DictNode]) -> Tuple[Dict[str, Dict[str, Any]], bool]:
    """hclify resource if pre-conditions met.

    :param resource: tf planned_values resource block
    :param conf: tf configuration resource block

    :returns:
        - resource_block: a list of strings representing the header columns
        - prepared: whether conditions met to prepare data
    """

    resource_block: Dict[str, Dict[str, Any]] = {}
    resource_block[resource["type"]] = {}
    prepared = False
    mode = ""
    if "mode" in resource:
        mode = resource.get("mode")
    # Rare cases where data block appears in resources with same name as resource block and only partial values
    # and where *_module resources don't have values field
    if mode == "managed" and "values" in resource:
        expressions = conf.get("expressions") if conf else None
        resource_block[resource["type"]][resource.get("name", "default")] = _hclify(resource["values"], expressions)
        resource_block[resource["type"]][resource.get("name", "default")]["__address__"] = resource.get("address")
        prepared = True
    return resource_block, prepared


def _find_child_modules(child_modules: ListNode) -> List[Dict[str, Dict[str, Any]]]:
    """
    Find all child modules if any. Including any amount of nested child modules.
    :type: child_modules: list of tf child_module objects
    :rtype: resource_blocks: list of hcl resources
    """
    resource_blocks = []
    for child_module in child_modules:
        if child_module.get("child_modules", []):
            nested_child_modules = child_module.get("child_modules", [])
            nested_blocks = _find_child_modules(nested_child_modules)
            for resource in nested_blocks:
                resource_blocks.append(resource)
        for resource in child_module.get("resources", []):
            resource_block, prepared = _prepare_resource_block(resource, None)
            if prepared is True:
                resource_blocks.append(resource_block)
    return resource_blocks


def parse_tf_plan(tf_plan_file: str, out_parsing_errors: Dict[str, str]) -> Tuple[Optional[Dict[str, Any]], Optional[List[Tuple[int, str]]]]:
    """
    :type tf_plan_file: str - path to plan file
    :rtype: tf_definition dictionary and template_lines of the plan file
    """
    tf_definition: Dict[str, Any] = {"resource": []}
    template, template_lines = parse(tf_plan_file, out_parsing_errors)
    if not template:
        return None, None
    for resource in template.get("planned_values", {}).get("root_module", {}).get("resources", []):
        conf = next(
            (
                x
                for x in template.get("configuration", {}).get("root_module", {}).get("resources", [])
                if x["type"] == resource["type"] and x["name"] == resource["name"]
            ),
            None,
        )
        resource_block, prepared = _prepare_resource_block(resource, conf)
        if prepared is True:
            tf_definition["resource"].append(resource_block)
    child_modules = template.get("planned_values", {}).get("root_module", {}).get("child_modules", [])
    # Terraform supports modules within modules so we need to search
    # in nested modules to find all resource blocks
    resource_blocks = _find_child_modules(child_modules)
    for resource in resource_blocks:
        tf_definition["resource"].append(resource)
    return tf_definition, template_lines


def _clean_simple_type_list(value_list: List[Any]) -> List[Any]:
    """
    Given a list of simple types return a cleaned list of simple types.
    Converts booleans that are input as strings back to booleans to maintain consistent expectations for later evaluation.
    Sometimes Terraform Plan will output Map values as strings regardless of boolean input.
    """
    for i in range(len(value_list)):
        if isinstance(value_list[i], str):
            lower_case_value = value_list[i].lower()
            if lower_case_value == "true":
                value_list[i] = True
            if lower_case_value == "false":
                value_list[i] = False         
    return value_list



#
# No change required normally except CheckCategories.XXXXX line 13
#
from __future__ import annotations

from collections.abc import Iterable

from checkov.common.checks.base_check import BaseCheck
from checkov.common.models.enums import CheckCategories

# Change the class name to your runner
# Eg. BaseXXXXXXXXXXXXXCheck
class BaseExampleRunnerCheck(BaseCheck):
    def __init__(
        self, name: str, id: str, supported_entities: Iterable[str], block_type: str, path: str | None = None
    ) -> None:
        # Set category for new checks
        # Look at checkov/common/models/enums.py for options
        categories = (CheckCategories.SUPPLY_CHAIN,)

        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=supported_entities,
            block_type=block_type,
        )
        self.path = path



from typing import Dict, Any

from checkov.common.models.enums import CheckResult
from checkov.kubernetes.checks.resource.base_container_check import BaseK8sContainerCheck
from checkov.kubernetes.checks.resource.k8s.k8s_check_utils import extract_commands


class ApiServerEncryptionProviders(BaseK8sContainerCheck):
    def __init__(self) -> None:
        id = "CKV_K8S_104"
        name = "Ensure that encryption providers are appropriately configured"
        super().__init__(name=name, id=id)

    def scan_container_conf(self, metadata: Dict[str, Any], conf: Dict[str, Any]) -> CheckResult:
        self.evaluated_container_keys = ["command"]
        keys, values = extract_commands(conf)

        if "kube-apiserver" in keys:
            if "--encryption-provider-config" not in keys:
                return CheckResult.FAILED

        return CheckResult.PASSED


check = ApiServerEncryptionProviders()



from typing import List, Any, Dict

from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.graph.checks_infra.solvers.base_solver import BaseSolver
from checkov.common.checks_infra.solvers.complex_solvers.base_complex_solver import BaseComplexSolver
from functools import reduce
from operator import and_


class AndSolver(BaseComplexSolver):
    operator = Operators.AND

    def __init__(self, solvers: List[BaseSolver], resource_types: List[str]) -> None:
        super().__init__(solvers, resource_types)

    def _get_operation(self, *args: Any, **kwargs: Any) -> Any:
        return reduce(and_, args)

    def get_operation(self, vertex: Dict[str, Any]) -> bool:
        for solver in self.solvers:
            if not solver.get_operation(vertex):
                return False
        return True



from typing import List, Any, Dict, Callable, Tuple

from networkx import DiGraph

from checkov.common.graph.checks_infra.enums import SolverType
from checkov.common.graph.checks_infra.solvers.base_solver import BaseSolver


class BaseFilterSolver(BaseSolver):
    def __init__(self, resource_types: List[str], attribute: str, value: Any) -> None:
        super().__init__(SolverType.FILTER)
        self.resource_types = resource_types
        self.attribute = attribute
        self.value = value
        self.vertices: List[Dict[str, Any]] = []

    def get_operation(self, *args: Any, **kwargs: Any) -> bool:
        raise NotImplementedError()

    def _get_operation(self, *args: Any, **kwargs: Any) -> Callable[..., bool]:
        raise NotImplementedError()

    def run(self, graph_connector: DiGraph) -> Tuple[List[Dict[str, Any]], List[Dict[str, Any]]]:
        raise NotImplementedError()



from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories

class ObjectStorageBucketPublicAccess(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure storage bucket does not have public access permissions."
        id = "CKV_YC_17"
        categories = [CheckCategories.GENERAL_SECURITY]
        supported_resources = ["yandex_storage_bucket"]
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
        )

    def scan_resource_conf(self, conf):
        if 'acl' in conf.keys():
            acl_block = conf['acl']
            if acl_block in [["public-read"], ["public-read-write"]]:
                return CheckResult.FAILED
        if 'grant' in conf.keys():
            grant_uri_block = conf['grant'][0]['uri']
            if grant_uri_block == ["http://acs.amazonaws.com/groups/global/AllUsers"]:
                return CheckResult.FAILED
        return CheckResult.PASSED

scanner = ObjectStorageBucketPublicAccess()


from abc import abstractmethod
from typing import Tuple, List, Dict, Any

from networkx import DiGraph

from checkov.common.graph.checks_infra.enums import SolverType


class BaseSolver:
    operator = ""

    def __init__(self, solver_type: SolverType) -> None:
        self.solver_type = solver_type

    @abstractmethod
    def get_operation(self, *args: Any, **kwargs: Any) -> Any:
        raise NotImplementedError()

    @abstractmethod
    def _get_operation(self, *args: Any, **kwargs: Any) -> Any:
        raise NotImplementedError()

    @abstractmethod
    def run(self, graph_connector: DiGraph) -> Tuple[List[Dict[str, Any]], List[Dict[str, Any]]]:
        raise NotImplementedError()

    @staticmethod
    def resource_type_pred(v: Dict[str, Any], resource_types: List[str]) -> bool:
        return not resource_types or ("resource_type" in v and v["resource_type"] in resource_types)



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class MemorystoreForRedisInTransitEncryption(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure Memorystore for Redis uses intransit encryption"
        id = "CKV_GCP_97"
        supported_resources = ['google_redis_instance']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "transit_encryption_mode"

    # Accounts for if key is present but is set to False
    def get_expected_value(self):
        return "SERVER_AUTHENTICATION"


check = MemorystoreForRedisInTransitEncryption()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.dockerfile.base_dockerfile_check import BaseDockerfileCheck

install_commands = [
    "install",
    "source-install",
    "reinstall",
    "groupinstall",
    "localinstall",
    "add",
]
update_commands = [
    "update",
    "--update"
]


class UpdateNotAlone(BaseDockerfileCheck):
    def __init__(self):
        name = "Ensure update instructions are not use alone in the Dockerfile"
        id = "CKV_DOCKER_5"
        supported_instructions = ["RUN"]
        categories = [CheckCategories.APPLICATION_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_instructions=supported_instructions)

    def scan_entity_conf(self, conf):
        update_instructions = []
        update_cnt = 0
        i = 0
        for instruction in conf:
            content = instruction['content']
            if instruction['instruction'] in self.supported_instructions:

                if any(x in content for x in update_commands):
                    update_cnt = update_cnt + 1
                    update_instructions.append(i)
                if any(x in content for x in install_commands):
                    update_cnt = update_cnt - 1
            i = i + 1

        if update_cnt <= 0:
            return CheckResult.PASSED, None
        output = []
        for i in update_instructions:
            output.append(conf[i])
            
        return CheckResult.FAILED, output


check = UpdateNotAlone()



import ast
import os

from flake8.options.manager import OptionManager

from flake8_plugins.flake8_class_attributes_plugin.flake8_class_attributes.checker import ClassAttributesChecker


def run_validator_for_test_file(filename, max_annotations_complexity=None,
                                strict_mode=False, attributes_order=None):
    test_file_path = os.path.join(
        os.path.dirname(os.path.abspath(__file__)),
        'test_files',
        filename,
    )
    with open(test_file_path, 'r') as file_handler:
        raw_content = file_handler.read()
    tree = ast.parse(raw_content)

    options = OptionManager('flake8_class_attributes_order', '0.1.3')
    options.use_class_attributes_order_strict_mode = strict_mode
    options.class_attributes_order = attributes_order
    ClassAttributesChecker.parse_options(options)

    checker = ClassAttributesChecker(tree=tree, filename=filename)
    if max_annotations_complexity:
        checker.max_annotations_complexity = max_annotations_complexity

    return list(checker.run())
